@@ -3,12 +3,15 @@ from typing import Dict, Any, List, Tuple, Callable as Function
 from .utils.JSDict import JSDict
 from replit import Database
 from datetime import datetime
-from datauri import parse
+
+# from datauri import parse
 from requests import post as _raw_post, get
 from .utils.switch import Switch
 from os import environ
 from .links import links
-from .utils.EventEmitter import BasicEventEmitter
+
+# from .utils.EventEmitter import EventEmitter
+from pyee import EventEmitter
 from time import sleep
 from threading import Thread as _Thread
 from .exceptions import InvalidSid
@@ -20,7 +23,7 @@ class module:
     pass
 
 
-class Track(BasicEventEmitter):
+class Track(EventEmitter):
     def __init__(self, func: Function[..., Any], ms: int) -> None:
         super()
         super().__init__()
@@ -52,7 +55,7 @@ class Track(BasicEventEmitter):
             self.t.join()
 
 
-class Client(BasicEventEmitter):
+class Client(EventEmitter):
     def __init__(self, sid: str) -> None:
         super()
         super().__init__()
@@ -297,7 +300,7 @@ class ReplManager:
         return res["replTitle"]
 
     def create(self, options: Dict[str, Any] = {}):
-        current = {"cache": True, "title": self.generateTitle(), "langauge": "nix"}
+        current = {"cache": True, "title": self.generateTitle(), "language": "nix"}
         current.update(options)
         options = current
         cache = options["cache"]
@@ -388,16 +391,16 @@ class PostManager:
 
 
 class NotificationManager:
-    def __init__(self, client) -> None:
+    def __init__(self, client: Client, seconds: int = 30) -> None:
         self.c = client
         self.cache = {}
-        self.track = Track(lambda: self.c.user.notifications.fetch_all(), 60)
+        self.track = Track(lambda: self.c.user.notifications.fetch_all(), seconds)
 
         def resolve_notifications(notifs) -> None:
             for i in notifs:
-                self.c.emit("notification", notifs[i])
-            if len(notifs) > 0:
-                self.markAsRead()
+                self.c.emit("notification", i, notifs[i])
+            # if len(notifs) > 0:
+            #     self.markAsRead()
 
         self.track.on("update", resolve_notifications)
 
@@ -749,7 +752,7 @@ class CurrentUser(User):
         image = options["image"]
         del options["image"]
         if image:
-            i = parse(image)
+            # i = parse(image)
             if i:
                 _headers = headers.copy()
                 _headers["Cookie"] = f"connect.sid={self.c.sid}"
@@ -1113,6 +1116,7 @@ class Comment:
         self.c = client
         self.comments = []
         self.parentComment = parent
+        self.id = None
 
     def update(self, data: Dict[str, Any]):
         comment_data = {
@@ -1180,15 +1184,20 @@ class Comment:
             if (getattr(options, "mention", False) and options.mention)
             else body
         )
+        if getattr(self, "parentComment", False):
+            id = self.parentComment.id
+        else:
+            id = self.id
         res = post(
             self.c.sid,
             "sendReplCommentReply",
-            vars={"input": {"body": send_text, "replCommentId": self.id}},
+            vars={"input": {"body": send_text, "replCommentId": id}},
         )
         if not res:
             return None
         comment = Comment(self.c)
         comment.update(res["createReplCommentReply"])
         if options.cache:
-            self.c.comments.cache[comment.id] = comment
+            if comment.id != None:
+                self.c.comments.cache[comment.id] = comment
         return comment
